@@ -1,17 +1,14 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 
-from builtins import str
 from builtins import map
-import locale
 import os
-import sys
 from simple_idml.indesign import indesign
 from simple_idml.commands import InDesignSoapCommand
 
 
 class InDesignSaveAsCommand(InDesignSoapCommand):
-    description = """SOAP call to a InDesignServer to save a file in (an)other format(s).
+    description = """SOAP call to an InDesign Server to save a file in other formats.
 
    PDF export parameters
    '''''''''''''''''''''
@@ -55,22 +52,20 @@ class InDesignSaveAsCommand(InDesignSoapCommand):
 
     def __init__(self):
         super(InDesignSaveAsCommand, self).__init__()
-        self.parser.add_argument('source', metavar='SOURCE', help="path/to/file.indd")
+        self.parser.add_argument('source', metavar='SOURCE', help="path/to/file.{indd|zip}")
         self.parser.add_argument('destinations', metavar='DESTINATIONS', help="file1|opt1=a,opt2=b;file2|opt1=c")
 
-    def parse_options(self):
-        encoding = locale.getpreferredencoding()
-        for i, a in enumerate(sys.argv):
-            sys.argv[i] = str(a.decode(encoding))
-        self.args = self.parser.parse_args()
-
     def execute(self):
         super(InDesignSaveAsCommand, self).execute()
         destinations = self.args.destinations.split(";")
         formats = map(self.parse_destination_arg, destinations)
-        responses = indesign.save_as(self.args.source, formats, self.args.url, self.args.client_workdir,
-                                     self.args.server_workdir, self.args.server_path_style,
-                                     not self.args.no_clean_workdir, self.ftp_params)
+        if os.path.splitext(self.args.source)[1] == ".zip":
+            func = indesign.export_package_as
+        else:
+            func = indesign.save_as
+        responses = func(self.args.source, formats, self.args.url, self.args.client_workdir,
+                         self.args.server_workdir, self.args.server_path_style,
+                         not self.args.no_clean_workdir, self.ftp_params)
         for response, destination in zip(responses, destinations):
             self.save_as(response, destination)
 
@@ -91,7 +86,7 @@ class InDesignSaveAsCommand(InDesignSoapCommand):
     def save_as(self, response, dst):
         if not response:
             return
-        with open(dst.split("|")[0], mode="w+") as fobj:
+        with open(dst.split("|")[0], mode="wb+") as fobj:
             fobj.write(response)
 
 
