@@ -1,6 +1,7 @@
 # -*- coding: utf-8 -*-
 
 import copy
+import datetime
 import os
 import re
 from decimal import Decimal
@@ -64,7 +65,7 @@ class IDMLXMLFile(object):
         if self._fobj is None:
             if self.working_copy_path:
                 filename = os.path.join(self.working_copy_path, self.name)
-                fobj = open(filename, mode="r+")
+                fobj = open(filename, mode="rb+")
             else:
                 fobj = self.idml_package.open(self.name, mode="r")
             self._fobj = fobj
@@ -73,7 +74,14 @@ class IDMLXMLFile(object):
     @property
     def dom(self):
         if self._dom is None:
-            dom = etree.fromstring(self.fobj.read())
+            xml = self.fobj.read()
+            try:
+                dom = etree.fromstring(xml, parser=etree.XMLParser(huge_tree=True))
+            except ValueError:
+                # Python3: when the fobj come from Story.create()
+                # it is strictly a textfile that cannot be implicitly
+                # read as a bytestring (required by etree.fromstring()).
+                dom = etree.fromstring(xml.encode('utf-8'))
             self._dom = dom
             self._fobj.close()
             self._fobj = None
@@ -106,7 +114,7 @@ class IDMLXMLFile(object):
         self._fobj = None
 
         # Must instanciate with a working_copy to use this.
-        fobj = open(os.path.join(self.working_copy_path, self.name), mode="w+")
+        fobj = open(os.path.join(self.working_copy_path, self.name), mode="wb+")
         fobj.write(self.tostring())
         fobj.close()
 
@@ -257,7 +265,7 @@ class Spread(IDMLXMLFile):
         last_page.set_face(face_required)
 
     def clear(self):
-        items = self.node.items()
+        items = list(self.node.items())
         self.node.clear()
         for k, v in items:
             self.node.set(k, v)
@@ -336,19 +344,15 @@ class Story(IDMLXMLFile):
         filename = os.path.join(working_copy_path, story_name)
         story._fobj = open(filename, mode="w+")
         story.fobj.write(
-            """<?xml version='1.0' encoding='UTF-8' standalone='yes'?>
+            f"""<?xml version='1.0' encoding='UTF-8' standalone='yes'?>
    <idPkg:Story xmlns:idPkg="http://ns.adobe.com/AdobeInDesign/idml/1.0/packaging" DOMVersion="7.5">
-     <Story Self="%(story_id)s" AppliedTOCStyle="n" TrackChanges="false" StoryTitle="$ID/" AppliedNamedGrid="n">
+     <Story Self="{story_id}" AppliedTOCStyle="n" TrackChanges="false" StoryTitle="$ID/" AppliedNamedGrid="n">
        <StoryPreference OpticalMarginAlignment="false" OpticalMarginSize="12" FrameType="TextFrameType" StoryOrientation="Horizontal" StoryDirection="LeftToRightDirection"/>
        <InCopyExportOption IncludeGraphicProxies="true" IncludeAllResources="false"/>
-       <XMLElement Self="%(xml_element_id)s" MarkupTag="XMLTag/%(xml_element_tag)s" XMLContent="%(story_id)s" />
+       <XMLElement Self="{xml_element_id}" MarkupTag="XMLTag/{xml_element_tag}" XMLContent="{story_id}" />
      </Story>
 </idPkg:Story>
-""" % {
-            "story_id": story_id,
-            "xml_element_tag": xml_element_tag,
-            "xml_element_id": xml_element_id
-        })
+""")
 
         story.fobj.close()
         story._fobj = None
@@ -367,9 +371,27 @@ class Story(IDMLXMLFile):
 
     def set_element_content(self, element_id, content):
         self.clear_element_content(element_id)
-        element = self.get_element_by_id(element_id)
-        xml_element = XMLElement(element=element)
+        xml_element = self.get_element_by_id(element_id)
         xml_element.set_content(content)
+        self._fix_siblings_style(xml_element)
+
+    def _fix_siblings_style(self, xml_element):
+        """Fix ticket #11 when importing XML."""
+        # Get the sibling elts that need to be styled.
+        siblings = []
+        for elt in xml_element.itersiblings():
+            if elt.tag not in ["Content", "Br"]:
+                break
+            siblings.append(elt)
+
+        if not len(siblings):
+            return
+
+        # The parent style is locally applied.
+        local_style = xml_element.clone_style_range()
+        for sibling in siblings:
+            local_style.append(sibling)
+        xml_element.addnext(local_style)
 
     def clear_element_content(self, element_id):
         element = self.get_element_by_id(element_id)
@@ -418,7 +440,8 @@ class Story(IDMLXMLFile):
 
     def remove_children(self, element_id, synchronize=False):
         elt = self.get_element_by_id(element_id).element
-        map(lambda c: elt.remove(c), elt.iterchildren())
+        for child in elt.iterchildren():
+            elt.remove(child)
         if synchronize:
             self.synchronize()
 
@@ -432,6 +455,19 @@ class Story(IDMLXMLFile):
         xml_element = XMLElement(element=element)
         xml_element.add_content(content, parent)
 
+    def add_note(self, element_id, note, author, when=None):
+        element = self.get_element_by_id(element_id)
+        when = when or datetime.datetime.now().replace(microsecond=0)
+        when = when.isoformat()
+        note_node = etree.fromstring(f"""<Note Collapsed="false" CreationDate="{when}" ModificationDate="{when}" UserName="{author}">
+                                            <ParagraphStyleRange AppliedParagraphStyle="ParagraphStyle/$ID/[No paragraph style]">
+                                                <CharacterStyleRange AppliedCharacterStyle="CharacterStyle/$ID/[No character style]">
+                                                    <Content>{note}</Content>
+                                                </CharacterStyleRange>
+                                            </ParagraphStyleRange>
+                                        </Note>""")
+        element.insert(0, note_node)
+
 
 class BackingStory(Story):
     def __init__(self, idml_package, name=BACKINGSTORY, working_copy_path=None):
@@ -444,7 +480,7 @@ class BackingStory(Story):
 
 class Designmap(IDMLXMLFile):
     name = "designmap.xml"
-    doctype = u'<?aid style="50" type="document" readerVersion="6.0" featureSet="257" product="7.5(142)" ?>'
+    doctype = '<?aid style="50" type="document" readerVersion="6.0" featureSet="257" product="7.5(142)" ?>'
     page_start_attr = "PageStart"
 
     def __init__(self, idml_package, working_copy_path):
@@ -625,7 +661,7 @@ class StyleMapping(IDMLXMLFile):
         try:
             super(StyleMapping, self).dom
         except AttributeError:
-            self._dom = etree.fromstring(self.initial_dom)
+            self._dom = etree.fromstring(self.initial_dom.encode("utf-8"))
         return self._dom
 
     @property
@@ -826,7 +862,7 @@ class XMLElement(Proxy):
         content_element = etree.Element("Content")
         content_element.text = content
         if style_range_node is None:
-            style_range_node = self._create_style_range_from_parent(parent)
+            style_range_node = parent.clone_style_range()
         style_range_node.append(content_element)
         self.element.append(style_range_node)
 
@@ -836,13 +872,10 @@ class XMLElement(Proxy):
         except IndexError:
             return
         # Ticket #8 - Fix the style locally.
-        local_style = self.get_local_character_style_range()
-        if local_style is None:
-            super_style = self.get_super_character_style_range()
-            if super_style is not None:
-                # Force super style locally.
-                local_style = self._create_style_range_from_parent(self)
-                self.apply_style_locally(local_style)
+        if self.get_local_character_style_range() is None and \
+           self.get_super_character_style_range() is not None:
+            local_style = self.clone_style_range()
+            self.apply_style_locally(local_style)
 
     def apply_style_locally(self, style_range_node):
         """The content node of self is moved into style_range_node. """
@@ -850,22 +883,40 @@ class XMLElement(Proxy):
         style_range_node.append(content_node)
         self.append(style_range_node)
 
-    # TODO: Why not calling this method directly on parent (which is a XMLElement too)?
-    def _create_style_range_from_parent(self, parent):
-        parent_style_node = parent.get_character_style_range()
-        applied_style = parent_style_node.get("AppliedCharacterStyle")
+    def clone_style_range(self):
+        style_node = self.get_character_style_range()
+        applied_style = style_node.get("AppliedCharacterStyle")
         style_range_node = etree.Element("CharacterStyleRange", AppliedCharacterStyle=applied_style)
         properties_node = etree.SubElement(style_range_node, "Properties")
 
-        for attr in ("PointSize", "FontStyle", "HorizontalScale", "Tracking", "FillColor", "Capitalization"):
-            if parent_style_node.get(attr) is not None:
-                style_range_node.set(attr, parent_style_node.get(attr))
+        attrs = [
+            "PointSize",
+            "FontStyle",
+            "HorizontalScale",
+            "Tracking",
+            "FillColor",
+            "FillTint",
+            "Capitalization",
+            "PointSize",
+            "StrokeWeight",
+            "MiterLimit",
+            "RubyFontSize",
+            "KentenFontSize",
+            "DiacriticPosition",
+            "Ligatures",
+            "OTFContextualAlternate",
+            "BaselineShift",
+        ]
+
+        for attr in attrs:
+            if style_node.get(attr) is not None:
+                style_range_node.set(attr, style_node.get(attr))
 
         for attr in ("Leading", "AppliedFont"):
             path = "Properties/%s" % attr
-            parent_attr_node = parent_style_node.find(path)
-            if parent_attr_node is not None:
-                properties_node.append(copy.deepcopy(parent_attr_node))
+            attr_node = style_node.find(path)
+            if attr_node is not None:
+                properties_node.append(copy.deepcopy(attr_node))
         return style_range_node
 
     def get_attribute(self, name):
